import sys
import os
import os.path
import shutil
import subprocess
import platform
import nose
from unittest import TestCase
from nose.plugins import Plugin
from nose.tools import with_setup
from distutils.sysconfig import get_config_var

if platform.python_version() < "2.5":
    def check_call(*args, **kwargs):
        ret = subprocess.call(*args, **kwargs)
        if ret != 0:
            cmd = kwargs.get('args', args[0])
            raise Exception("command '%s' returned %s" % (cmd, ret))
    subprocess.check_call = check_call

here = os.getcwd()
exe_ext = get_config_var("EXE")

compiler = None
debug = False
do_clean = False


def pybuild():
    pybuild_cmds = [sys.executable, "setup.py", "build"]
    if compiler is not None:
        pybuild_cmds.append("--compiler="+compiler)
    subprocess.check_call(pybuild_cmds)


class OurPlugin(Plugin):
    def options(self, parser, env=os.environ):
        parser.add_option("--compiler", dest="compiler")
        parser.add_option("--clean", action="store_true", dest="clean")
        parser.add_option('--d-debug', action="store_true", dest="debug")

    def configure(self, options, conf):
        global do_clean, compiler, debug
        debug = options.debug

        if options.compiler:
            compiler = options.compiler
        if options.clean:
            do_clean = True


def setup():
    os.chdir(here)


def teardown():
    pass


def build_and_run():
    if do_clean:
        if os.path.exists("build"):
            shutil.rmtree("build")
        return
    pybuild()
    subprocess.check_call([sys.executable, "test.py"])


@with_setup(setup, teardown)
def test_hello():
    os.chdir("examples")
    os.chdir("hello")
    build_and_run()


@with_setup(setup, teardown)
def test_many_libs():
    os.chdir("tests")
    os.chdir("many_libs")
    build_and_run()


@with_setup(setup, teardown)
def test_arraytest():
    os.chdir("examples")
    os.chdir("arraytest")
    build_and_run()


@with_setup(setup, teardown)
def test_inherit():
    os.chdir("examples")
    os.chdir("inherit")
    build_and_run()


@with_setup(setup, teardown)
def test_rawexample():
    os.chdir("examples")
    os.chdir("rawexample")
    build_and_run()


@with_setup(setup, teardown)
def test_testdll():
    os.chdir("examples")
    os.chdir("testdll")
    build_and_run()


@with_setup(setup, teardown)
def test_d_and_c():
    os.chdir("examples")
    os.chdir("misc")
    os.chdir("d_and_c")
    build_and_run()


@with_setup(setup, teardown)
def test_multithreading():
    os.chdir("tests")
    os.chdir("multithreading")
    build_and_run()


def build_pydexe():
    cmds = [sys.executable, "setup.py", "pydexe"]
    if compiler is not None:
        cmds.append("--compiler="+compiler)
    if debug:
        cmds.append("-g")
    subprocess.check_call(cmds)


def remove_exe(cmd):
    if os.path.exists(cmd + exe_ext):
        os.remove(cmd+exe_ext)


def build_and_run_pydexe(nom):
    if do_clean:
        if os.path.exists("build"):
            shutil.rmtree("build")
        remove_exe(nom)
        return
    build_pydexe()
    subprocess.check_call([os.path.join(".", nom + exe_ext)])


@with_setup(setup, teardown)
def test_extra():
    os.chdir("tests")
    os.chdir("pyd_unittests")
    os.chdir("extra")
    build_and_run_pydexe("extra")


class ErrorTests(TestCase):
    def setUp(self):
        setup();
        os.chdir("tests")
        os.chdir("errors")

    def test_error1(self):
        os.chdir("error1")
        if do_clean:
            if os.path.exists("build"):
                shutil.rmtree("build")
            return
        pybuild_cmds = [sys.executable, "setup.py", "build"]
        if compiler is not None:
            pybuild_cmds.append("--compiler="+compiler)
        proc = subprocess.Popen(pybuild_cmds, stderr=subprocess.PIPE)
        stdout, stderr = proc.communicate()
        stderr = stderr.decode('utf-8')
        assert "Test: Cannot find constructor with params (int) among" in stderr 
        assert "Test(immutable(int))" in stderr 

class PydUnittests(TestCase):
    def setUp(self):
        setup()
        os.chdir("tests")
        os.chdir("pyd_unittests")

    def tearDown(self):
        teardown()

    def test_class_wrap(self):
        os.chdir("class_wrap")
        build_and_run_pydexe("class_wrap")

    def test_def(self):
        os.chdir("def")
        build_and_run_pydexe("def")

    def test_embedded(self):
        os.chdir("embedded")
        build_and_run_pydexe("embedded")

    def test_make_object(self):
        os.chdir("make_object")
        build_and_run_pydexe("make_object")

    def test_pydobject(self):
        os.chdir("pydobject")
        build_and_run_pydexe("pydobject")

    def test_struct_wrap(self):
        os.chdir("struct_wrap")
        build_and_run_pydexe("struct_wrap")

    def test_const(self):
        os.chdir("const")
        build_and_run_pydexe("const")

    def test_typeinfo(self):
        os.chdir("typeinfo")
        build_and_run_pydexe("typeinfo")

    def test_func_wrap(self):
        os.chdir("func_wrap")
        build_and_run_pydexe("func_wrap")

    def test_thread(self):
        os.chdir("thread")
        build_and_run_pydexe("thread")


class DeimosUnittests(TestCase):
    def setUp(self):
        setup()
        os.chdir("tests")
        os.chdir("deimos_unittests")

    def tearDown(self):
        teardown()

    def test_link(self):
        os.chdir('link')
        build_and_run_pydexe("link")

    def test_object(self):
        os.chdir('object_')
        build_and_run_pydexe("object_")

    def test_datetime(self):
        os.chdir('datetime')
        build_and_run_pydexe("datetime")


@with_setup(setup, teardown)
def test_pyind():
    os.chdir('examples')
    os.chdir('pyind')
    build_and_run_pydexe("pyind")


@with_setup(setup, teardown)
def test_simple_embedded():
    os.chdir('examples')
    os.chdir('simple_embedded')
    build_and_run_pydexe("hello")


@with_setup(setup, teardown)
def test_interpcontext():
    os.chdir('examples')
    os.chdir('interpcontext')
    build_and_run_pydexe("interpcontext")


@with_setup(setup, teardown)
def test_def():
    os.chdir('examples')
    os.chdir('def')
    build_and_run()


@with_setup(setup, teardown)
def test_pydobject():
    os.chdir('examples')
    os.chdir('pydobject')
    build_and_run_pydexe("example")


nose.main(addplugins=[OurPlugin()])



import os, os.path
import itertools
import sys
import re


FORBIDDEN_EXTENSIONS = [
    '.pyc', '.pyo', # Python bytecode
    '.marks', # jEdit bookmark files
    '.map', # Created automatically by the DMD compiler; needn't distribute.
    '.swp', # Vim swap files
  ]

FORBIDDEN_DIRECTORIES = [
    lambda d: d.lower() in ('.svn', 'cvs', 'build', 'dist', '.hg', '.git', '.tup'),
    lambda d: d.startswith('__'),
  ]

INCLUDE_ONLY_IN_SOURCE_DISTRIBUTION = [
    'build_manifest.py',
    'setup.py',
  ]

def split_complete(path):
    folders = []
    while True:
        path, p2 = os.path.split(path)
        if p2 != '':
            folders.append(p2)
        else:
            if path!='':
                folders.append(path)
            break
    return list(reversed(folders))
def include_path(path):
    pathsubs = split_complete(path)
    assert pathsubs
    filebase, ext = os.path.splitext(pathsubs[-1])
    if pathsubs[0] == 'infrastructure':
        if pathsubs == ['infrastructure','pyd','LICENSE']:
            return True
        if pathsubs == ['infrastructure','d','python_dll_def.def_template']:
            return True
        if pathsubs == ['infrastructure','d','so_ctor.c']:
            return True
        if pathsubs == ['infrastructure','python','python.d']:
            return False
        if ext.lower() == '.d':
            return True
        if pathsubs[0:2] == ['infrastructure','windows'] and \
                re.match("python.._digitalmars\\.lib",pathsubs[-1]):
            return True
        return False
    if len(pathsubs) == 1 and ext.lower() == '.py':
        return True
    if len(pathsubs) == 1 and pathsubs[0] == "version.txt":
        return True
    return False


def buildManifest(outputStream, isForSourceDist):
    includedPaths, excludedPaths = listFiles(isForSourceDist)
    for path in includedPaths:
        # print >> outputStream, 'include "%s"' % convertPathToDistutilsStandard(path)
        outputStream.write(convertPathToDistutilsStandard(path))
        outputStream.write("\n")


def convertPathToDistutilsStandard(path):
    return path.replace(os.sep, '/')


def listFiles(isForSourceDist):
    curDirAndSep = os.curdir + os.sep

    includedPaths = []
    excludedPaths = []
    for rootPath, dirs, files in os.walk(os.curdir):
        if rootPath.startswith(os.curdir + os.sep):
            rootPath = rootPath[len(os.curdir + os.sep):]
        elif rootPath.startswith(os.curdir):
            rootPath = rootPath[len(os.curdir):]

        # The os.walk interface specifies that destructively modifying dirs
        # will influence which subdirs are visited, so we determine which
        # subdirs are forbidden and remove them from dirs.
        for subDir in dirs[:]:
            for filterFunc in FORBIDDEN_DIRECTORIES:
                if filterFunc(subDir):
                    dirs.remove(subDir)

        for f in sorted(files):
            fPath = os.path.join(rootPath, f)
            if os.path.splitext(f)[1].lower() in FORBIDDEN_EXTENSIONS:
                excludedPaths.append(fPath)
            else:
                includedPaths.append(fPath)

    if not isForSourceDist:
        for path in INCLUDE_ONLY_IN_SOURCE_DISTRIBUTION:
            if path in includedPaths:
                includedPaths.remove(path)
                excludedPaths.append(path)

    excludedPaths.extend([path for path in includedPaths if not include_path(path)])
    includedPaths = [path for path in includedPaths if include_path(path)]

    return includedPaths, excludedPaths


if __name__ == '__main__':
    import sys
    buildManifest(sys.stdout, True)



import json
import six
import sys
import platform
import os.path
from collections import OrderedDict
from pyd.patch_distutils import new_compiler
from distutils.command.build_ext import build_ext
from distutils.dist import Distribution

help = "specify compiler (dmd, ldc, or gdc)"
ext_help = "specify we are building a python extension"
default = 'dmd'

if six.PY2:
    from optparse import OptionParser
    parser = OptionParser()
    parser.add_option("--compiler", dest="compiler", default=default, help=help)
    parser.add_option("--as-extension", dest="extension", default=False, help=ext_help)
    (options, args) = parser.parse_args()
else:
    from argparse import ArgumentParser
    parser = ArgumentParser("""Generates dub configurations for the python instance that invokes this script.
    """)
    parser.add_argument("--compiler", default=default, help=help)
    parser.add_argument("--as-extension", dest="extension", action='store_true', default=False, help=ext_help)
    options = parser.parse_args()

compiler = new_compiler(options.compiler)
compiler.build_exe = not options.extension

class MockExt:
    def __init__(self):
        self.libraries = []
ext = build_ext(Distribution())

libraries = ext.get_libraries(MockExt())
lib_file = compiler._lib_file([])
if lib_file:
    lib_file = os.path.basename(lib_file).replace(compiler.static_lib_extension, "")
    if sys.platform == 'win32':
        libraries.append(os.path.join('$PYD_PACKAGE_DIR','infrastructure', 'windows', lib_file))
if __name__ == '__main__':
    config =  OrderedDict()
    config['name'] = "python%s%s" % platform.python_version_tuple()[0:2]
    config['versions'] = compiler.all_versions()
    config['libs'] = libraries

    print (json.dumps(config, indent=4))



__all__ = ('setup', 'Extension')

from pyd import patch_distutils # Cause distutils to be hot-patched.
import sys, os, os.path

from distutils.core import setup as base_setup, Extension as std_Extension, Command
from distutils.util import get_platform
from distutils.dep_util import newer_group
from distutils import log
from distutils.errors import DistutilsOptionError

class Extension(std_Extension):
    def __init__(self, *args, **kwargs):
        if 'define_macros' in kwargs or 'undef_macros' in kwargs:
            raise DistutilsOptionError('D does not support macros, so the'
                ' "define_macros" and "undef_macros" arguments are not'
                ' supported.  Instead, consider using the "Version Condition"'
                ' and "Debug Condition" conditional compilation features'
                ' documented at http://www.digitalmars.com/d/version.html'
                '\n  Version flags can be passed to the compiler via the'
                ' "version_flags" keyword argument to DExtension; debug flags'
                ' via the "debug_flags" keyword argument.  For example, when'
                ' used with the DMD compiler,'
                '\n    DExtension(..., version_flags=["a", "b"])'
                '\nwill cause'
                '\n    -version=a -version=b'
                '\nto be passed to the compiler.'
              )

        # If the user has requested any version_flags or debug_flags, we use
        # the distutils 'define_macros' keyword argument to carry them (they're
        # later unpacked in the dcompiler module).

        self.version_flags = kwargs.pop('version_flags',[])
        self.debug_flags = kwargs.pop('debug_flags',[])

        if 'raw_only' in kwargs:
            kwargs['with_pyd'] = False
            kwargs['with_main'] = False
            del kwargs['raw_only']
        self.with_pyd  = kwargs.pop('with_pyd', True)
        self.build_deimos = kwargs.pop('build_deimos', False)
        self.with_main = kwargs.pop('with_main', True)
        self.pyd_optimize = kwargs.pop('optimize', False)
        self.d_unittest = kwargs.pop('d_unittest', False)
        self.d_property = kwargs.pop('d_property', False)
        self.d_lump = kwargs.pop('d_lump', False)
        self.string_imports = kwargs.pop('string_imports', [])
        if self.with_main and not self.with_pyd:
            # The special PydMain function should only be used when using Pyd
            self.with_main = False

        std_Extension.__init__(self, *args, **kwargs)


class build_pyd_embedded_exe(Command):
    description = "Build a D application that embeds python with Pyd"

    user_options = [
        ('compiler=', 'c',
         "specify the compiler type"),
        ("optimize", "O", "Ask the D compiler to optimize the generated code, at the expense of"
                " safety features such as array bounds checks."),
        ('debug', 'g',
         "compile extensions and libraries with debugging information"),
        ('force', 'f',
         "forcibly build everything (ignore file timestamps)"),
        ("print-flags", None, "Don't build, just print out version flags for pyd") ]

    boolean_options = ['print-flags', 'debug', 'force']

    def initialize_options(self):
        self.print_flags = False
        self.compiler = None
        self.build_temp = None
        self.build_lib = None
        self.build_platlib = None
        self.build_base = "build"
        self.include_dirs = None
        self.libraries = None
        self.library_dirs = None
        self.link_objects = None
        self.debug = None
        self.optimize = 0
        self.dry_run = 0
        self.verbose = 0
        self.force = 0

    def finalize_options(self):
        self.extensions = self.distribution.ext_modules
        plat_specifier = ".%s-%s" % (get_platform(), sys.version[0:3])
        if self.build_platlib is None:
            self.build_platlib = os.path.join(self.build_base, 'lib' + plat_specifier)
        if self.build_lib is None:
            self.build_lib = self.build_platlib
        if self.build_temp is None:
            self.build_temp = os.path.join(self.build_base, 'temp' + plat_specifier)

    def run(self):
        # mostly copied from distutils.command.build_ext
        from distutils.ccompiler import new_compiler
        if not self.extensions:
            return
        self.compiler = new_compiler(
                compiler=self.compiler or patch_distutils.get_default_compiler(),
                verbose=self.verbose,
                dry_run=self.dry_run,
                force=self.force)
        from pyd import dcompiler
        assert isinstance(self.compiler, dcompiler.DCompiler)
        self.compiler.build_exe = True
        self.compiler.optimize = self.optimize
        # irrelevant for D compilers?
        #customize_compiler(self.compiler)
        if self.include_dirs is not None:
            self.compiler.set_include_dirs(self.include_dirs)
        if self.libraries is not None:
            self.compiler.set_libraries(self.libraries)
        if self.library_dirs is not None:
            self.compiler.set_library_dirs(self.library_dirs)
        if self.link_objects is not None:
            self.compiler.set_link_objects(self.link_objects)

        if self.print_flags:
            print( ' '.join(self.compiler.versionOpts()) )
        else:
            for ext in self.extensions:
                self.per_ext(ext)

    def per_ext(self, ext):
            self.compiler.init_d_opts(self, ext)
            # mostly copied from distutils.command.build_ext
            sources = ext.sources
            if sources is None or type(sources) not in (list, tuple):
                raise DistutilsSetupError(
                        ("in 'pydexe' option (extension '%s'), " +
                        "'sources' must be present and must be " +
                        "a list of source filenames") % ext.name)
            sources = list(sources)
            ext_path = self.get_ext_fullpath(ext.name)
            depends = sources + ext.depends
            if not (self.force or newer_group(depends, ext_path, 'newer')):
                log.debug("skipping '%s' extension (up-to-date)", ext.name)
                return
            else:
                log.info("building '%s' extension", ext.name)

            extra_args = ext.extra_compile_args or []
            macros = ext.define_macros[:]

            objects = self.compiler.compile(sources,
                    output_dir=self.build_temp,
                    macros=macros,
                    include_dirs=ext.include_dirs,
                    debug=self.debug,
                    extra_postargs=extra_args,
                    depends=ext.depends)
            self._built_objects = objects[:]
            if ext.extra_objects:
                objects.extend(ext.extra_objects)
            language = ext.language or self.compiler.detect_language(sources)

            self.compiler.link_executable(objects, ext_path,
                    libraries=self.get_libraries(ext),
                    library_dirs=ext.library_dirs,
                    runtime_library_dirs=ext.runtime_library_dirs,
                    extra_postargs=extra_args,
                    debug=self.debug,
                    target_lang=language)
            import shutil
            shutil.copy(self.compiler.executable_filename(ext_path), '.')

    def get_ext_fullpath(self, ext_name):
        fullname = ext_name
        modpath = fullname.split('.')
        filename = self.get_ext_filename(ext_name)
        filename = os.path.split(filename)[-1]

        filename = os.path.join(*modpath[:-1]+[filename])
        return os.path.join(self.build_lib, filename)

    def get_ext_filename(self, ext_name):
        from distutils.sysconfig import get_config_var
        ext_path = ext_name.split( ".")
        if os.name == "os2":
            old_path = ext_path[:]
            ext_path[len(ext_path)-1] = ext_path[len(ext_path)-1][:8]
            assert False, ("build_ext does this, so it probably works, but check it anyways! We are on OS2, and OS2" +
                    "does not permit file names of length > 8, so we are using %r instead of %r") % (old_path, ext_path)
        # link_executable does this for us
        # (override extension with compiler.exe_extension)
        #exe_ext = get_config_var("EXE")
        exe_ext = ''
        if os.name == "nt" and self.debug:
            return os.path.join(*ext_path) + "_d" + exe_ext
        return os.path.join(*ext_path) + exe_ext

    def get_libraries(self, ext):
        # mostly copied from build_ext.get_libraries
        if sys.platform == "win32":
            return ext.libraries
        elif sys.platform[:6] == "cygwin":
            return ext.libraries
        else:
            from distutils import sysconfig
            if sysconfig.get_config_var('Py_ENABLE_SHARED'):
                x = ''
                if hasattr(sys, 'abiflags'):
                    x = sys.abiflags
                pythonlib = "python{}.{}{}".format(
                        sys.hexversion >> 24,
                        (sys.hexversion >> 16) & 0xff,
                        x)
                if hasattr(sys,'pydebug') and sys.pydebug:
                    pythonlib += '_d'
                return ext.libraries + [pythonlib]
            else:
                return ext.libraries

def pydexe_sanity_check():
    import sys
    if sys.argv[1] != 'pydexe':
        print( "use pydexe, not %s" % sys.argv[1] )
        sys.exit(1)

def setup(*args, **kwargs):
    if 'cmdclass' not in kwargs:
        kwargs['cmdclass'] = {}
    kwargs['cmdclass']['pydexe'] = build_pyd_embedded_exe
    base_setup(*args, **kwargs)



# The distutils.ccompiler module doesn't allow the runtime addition of compiler
# classes that live outside the distutils package, and of course it doesn't
# recognize the D language and its associated compilers by default.
#
# This module hot-patches distutils.ccompiler to overcome those limitations.
#
# To apply these changes, the setup.py file for extension modules written in D
# should execute the following import statement:
#   from pyd import patch_distutils

from distutils import ccompiler as cc
from distutils.command import build, build_ext

from pyd import dcompiler

cc.CCompiler.language_map['.d'] = 'd'
cc.CCompiler.language_order.insert(0, 'd')

d_compilers = {
    'dmd': ('pyd.dcompiler', 'DMDDCompiler', 'Digital Mars D'),
    'gdc': ('pyd.dcompiler', 'GDCDCompiler', 'GCC D Compiler'),
    'ldc': ('pyd.dcompiler', 'LDCDCompiler', 'LLVM D Compiler'),
    'ldc2': ('pyd.dcompiler', 'LDCDCompiler', 'LLVM D Compiler'),
}

for compiler, compiler_class in d_compilers.items():
    cc.compiler_class[compiler] = compiler_class

_old_new_compiler = cc.new_compiler

def new_compiler(compiler=None, dry_run=0, force=0, **kwargs):
    if compiler is not None:
        compiler = compiler.lower()

    if compiler not in d_compilers:
        return _old_new_compiler(compiler=compiler,
            dry_run=dry_run, force=force, **kwargs
          )
    else:
        # non-lazy people should probably use import_module here
        DCompiler = getattr(dcompiler, d_compilers[compiler][1])
        return DCompiler(None, dry_run, force)

cc.new_compiler = new_compiler
#   python setup.py build --compiler=dmd
def get_default_compiler(*args, **kwargs):
    if dcompiler._isPlatWin:
        return 'dmd'
    else:
        return 'dmd'
# Force the distutils build command to recognize the '--optimize' or '-O'
# command-line option.
build.build.user_options.extend(
    [('optimize', 'O',
      'Ask the D compiler to optimize the generated code, at the expense of'
      ' safety features such as array bounds checks.'), ])
build.build.boolean_options.append('optimize')

_old_initialize_options = build.build.initialize_options
def _new_initialize_options(self):
    _old_initialize_options(self)
    self.optimize = 0
build.build.initialize_options = _new_initialize_options

# Force build commands to actually send optimize option to D compilers

_old_build_ext = build_ext.build_ext.build_extension

def reinit_compiler(ext):
    # build_ext.run initializes compiler then calls build_extensions.
    # we don't want that last part..
    old_build_exts = ext.build_extensions
    ext.compiler = None
    def durp(): pass
    ext.build_extensions = durp
    ext.build_extensions()
    try:
        ext.run()
    finally:
        ext.build_extensions = old_build_exts

def new_build_ext(self, ext):
    old_compiler = None
    lang = ext.language or self.compiler.detect_language(ext.sources)
    # we handle d default compiler here now
    if lang == 'd' and not isinstance(self.compiler, dcompiler.DCompiler):
        self.compiler = new_compiler('dmd')
    if lang != 'd' and isinstance(self.compiler, dcompiler.DCompiler):
        # probably user put --compiler=somedcompiler in the command line.
        # welp, lets try to rerun the compiler initializer for this extension
        old_compiler = self.compiler
        reinit_compiler(self)
    if isinstance(self.compiler, dcompiler.DCompiler):
        build = self.distribution.get_command_obj('build')
        self.compiler.init_d_opts(build, ext)
    _old_build_ext(self,ext)
    if old_compiler:
        self.compiler = old_compiler
build_ext.build_ext.build_extension = new_build_ext



# DSR:2005.10.27.23.51

# XXX:
# These two will have to wait until DMD can create shared libraries on Linux,
# because DSR doesn't have (the non-free version of) MSVC 2003, which is
# necessary to create a debug build or a UCS4 build of Python 2.4 on Windows:
# - Handle distutils debug builds responsibly (make sure both -debug and -g are
#   passed through to DMD, even if optimizations are requested).  Also make
#   sure that extensions built with this module work under debug builds of
#   Python.
# - Try out a UCS4 build of Python to make sure that works.

import os, os.path, sys

from distutils import ccompiler as cc
from distutils.sysconfig import get_config_var
from distutils.ccompiler import gen_lib_options
from distutils.errors import (
    DistutilsExecError, DistutilsFileError, DistutilsPlatformError,
    CompileError, LibError, LinkError, UnknownFileError
)

_isPlatCygwin = sys.platform.lower() == 'cygwin'

def winpath(path_, winonly):
    if _isPlatCygwin and winonly:
        from subprocess import Popen, PIPE
        stdout,_ = Popen(['cygpath', '-w', path_], stdout=PIPE).communicate()
        return stdout.strip()
    else:
        return path_
def cygpath(path_, winonly):
    if _isPlatCygwin and winonly:
        from subprocess import Popen, PIPE
        stdout,_ = Popen(['cygpath', path_], stdout=PIPE).communicate()
        return stdout.strip()
    else:
        return path_

def is_posix_static_python():
    if (sys.platform == "win32" or
       sys.platform[:6] == "cygwin"):
        return False
    else:
        return not get_config_var('Py_ENABLE_SHARED')
def posix_static_python_opts():
    ls = [l for l in get_config_var('LIBS').split(' ') if l]
    print(ls)
    ls.extend([l for l in get_config_var('MODLIBS').split(' ') if l])
    print(ls)
    return ls

def posix_static_python_lib():
    return os.path.join(get_config_var('LIBPL'),get_config_var('LIBRARY'))

_isPlatWin = sys.platform.lower().startswith('win') or _isPlatCygwin
_isPlatOSX = sys.platform.lower() == 'darwin'

_infraDir = os.path.join(os.path.dirname(__file__), 'infrastructure')

from pyd.pyd_support import make_pydmain, make_pyddef

_pydFiles = [
    'class_wrap.d',
    'ctor_wrap.d',
    'def.d',
    'embedded.d',
    'exception.d',
    'extra.d',
    'func_wrap.d',
    'make_object.d',
    'make_wrapper.d',
    'op_wrap.d',
    'pyd.d',
    'pydobject.d',
    'references.d',
    'struct_wrap.d',
    'thread.d',
]

_utilFiles = [
    'conv.d',
    'typeinfo.d',
    'typelist.d',
    'multi_index.d',
    'replace.d',
]

_deimosFiles = [
    'abstract_.d',
    'ast.d',
    'boolobject.d',
    'bufferobject.d',
    'bytearrayobject.d',
    'bytesobject.d',
    'cellobject.d',
    'ceval.d',
    'classobject.d',
    'cobject.d',
    'codecs.d',
    'code.d',
    'compile.d',
    'complexobject.d',
    'cStringIO.d',
    'datetime.d',
    'descrobject.d',
    'dictobject.d',
    'enumobject.d',
    'errcode.d',
    'eval.d',
    'fileobject.d',
    'floatobject.d',
    'frameobject.d',
    'funcobject.d',
    'genobject.d',
    'grammar.d',
    'import_.d',
    'intobject.d',
    'intrcheck.d',
    'iterobject.d',
    'listobject.d',
    'longintrepr.d',
    'longobject.d',
    'marshal.d',
    'memoryobject.d',
    'methodobject.d',
    'modsupport.d',
    'moduleobject.d',
    'node.d',
    'object.d',
    'objimpl.d',
    'parsetok.d',
    'pgenheaders.d',
    'pyarena.d',
    'pyatomic.d',
    'pycapsule.d',
    'pydebug.d',
    'pyerrors.d',
    'pymem.d',
    'pyport.d',
    'pystate.d',
    'pystrcmp.d',
    'pystrtod.d',
    'Python.d',
    'pythonrun.d',
    'pythread.d',
    'rangeobject.d',
    'setobject.d',
    'sliceobject.d',
    'stringobject.d',
    'structmember.d',
    'structseq.d',
    'symtable.d',
    'sysmodule.d',
    'timefuncs.d',
    'traceback.d',
    'tupleobject.d',
    'unicodeobject.d',
    'weakrefobject.d',
]

_pyVerXDotY = '.'.join(str(v) for v in sys.version_info[:2]) # e.g., '2.4'
_pyVerXY = _pyVerXDotY.replace('.', '') # e.g., '24'

def spawn0(self, cmdElements):
    import platform
    if platform.python_version() < "2.6":
        exec(
        """
try:
    self.spawn(cmdElements)
except DistutilsExecError, msg:
    raise CompileError(msg)""")
    else:
        exec('''
try:
            self.spawn(cmdElements)
except DistutilsExecError as msg:
            raise CompileError(msg)
''')

def wrapped_spawn(self, cmdElements, tag):
    '''
    wrap spawn with unique-ish travis fold prints
    '''
    import uuid
    a = uuid.uuid1()
    print("travis_fold:start:%s-%s" % (tag, a))
    try:
        spawn0(self, cmdElements)
    finally:
        print("travis_fold:end:%s-%s" % (tag, a))


class DCompiler(cc.CCompiler):

    src_extensions = ['.d']
    obj_extension = (_isPlatWin and '.obj') or '.o'
    static_lib_extension = (_isPlatWin and '.lib') or '.a'
    shared_lib_extension = (_isPlatWin and '.pyd') or '.so'
    static_lib_format = (_isPlatWin and '%s%s') or 'lib%s%s'
    shared_lib_format = '%s%s'
    exe_extension = (_isPlatWin and '.exe') or ''

    def __init__(self, *args, **kwargs):
        cc.CCompiler.__init__(self, *args, **kwargs)
        self.winonly = False
        self.proj_name = None
        self.build_exe = False
        self.with_pyd = True
        self.with_main = True
        self.build_deimos = False
        self.lump = False
        self.optimize = False
        self.version_flags_from_ext = []
        self.debug_flags_from_ext = []
        self.string_imports_from_ext = []
        # Get DMD/GDC/LDC specific info
        self._initialize()
        # _binpath
        try:
            dBin = os.environ[self._env_var]
            if not os.path.isfile(dBin):
                self.warn("Environment variable %s provided, but file '%s' does not exist." % (self._env_var, dBin))
                raise KeyError
        except KeyError:
            if _isPlatWin:
                # The environment variable wasn't supplied, so search the PATH.
                # Windows requires the full path for reasons that escape me at
                # the moment.
                for compiler in self.executables['compiler']:
                    if '.' not in compiler:
                        compiler += self.exe_extension
                    dBin = _findInPath(compiler)
                    if dBin: break
                if dBin is None:
                    raise DistutilsFileError('You must either set the %s'
                        ' environment variable to the full path of the %s'
                        ' executable, or place the executable on the PATH.' %
                        (self._env_var, self.executables['compiler'][0])
                    )
            else:
                # Just run it via the PATH directly in Linux
                dBin = self.executables['compiler'][0]
        self._binpath = dBin
        self._unicodeOpt = 'Python_Unicode_UCS' + ((sys.maxunicode == 0xFFFF and '2') or '4')

    def _initialize(self):
        # It is intended that this method be implemented by subclasses.
        raise NotImplementedError( "Cannot initialize DCompiler, use DMDDCompiler, GDCDCompiler or LDCDCompiler instead.")

    def init_d_opts(self, cmd, ext):
            self.optimize = cmd.optimize or ext.pyd_optimize
            self.with_pyd = ext.with_pyd
            self.with_main = ext.with_main
            self.build_deimos = ext.build_deimos
            self.lump = ext.d_lump
            self.proj_name = ext.name
            self.version_flags_from_ext = ext.version_flags
            self.debug_flags_from_ext = ext.debug_flags
            self.string_imports_from_ext = ext.string_imports
            self.unittest_from_ext = ext.d_unittest
            self.property_from_ext = ext.d_property


    def _def_file(self, output_dir, output_filename):
        """A list of options used to tell the linker how to make a dll/so. In
        DMD, it is the .def file. In GDC, it is
        ['-shared', '-Wl,-soname,blah.so'] or similar."""
        raise NotImplementedError( "Cannot initialize DCompiler, use DMDDCompiler, GDCDCompiler or LDCDCompiler instead.")

    def _lib_file(self, libraries):
        return ''

    def find_library_file(self, dirs, lib, debug=0):
        shared_f = self.library_filename(lib, lib_type='shared')
        static_f = self.library_filename(lib, lib_type='static')

        for dir in dirs:
            shared = os.path.join(dir, shared_f)
            static = os.path.join(dir, static_f)

            if os.path.exists(shared):
                return shared
            elif os.path.exists(static):
                return static

        return None

    def python_versions(self):
        optf = 'Python_%d_%d_Or_Later'
        def pv2(minor):
            ret = []
            if not self.build_exe:
                ret.append('PydPythonExtension')
            ret.extend([(optf % (2,m)) for m in range(4,minor+1)])
            return ret
        def pv3(minor):
            return [(optf % (3,m)) for m in range(0,minor+1)]
        major = sys.version_info[0]
        minor = sys.version_info[1]
        if major == 2: return pv2(minor)
        if major == 3: return  pv2(7) + pv3(minor)
        assert False, "what python version is this, anyways?"

    def all_versions(self):
        return self.python_versions() + [self._unicodeOpt]

    def versionOpts(self):
        # Python version option allows extension writer to take advantage of
        # Python/C API features available only in recent version of Python with
        # a version statement like:
        #   version(Python_2_4_Or_Later) {
        #     Py_ConvenientCallOnlyAvailableInPython24AndLater();
        #   } else {
        #     // Do it the hard way...
        #   }
        return [self._versionOpt % v for v in self.all_versions()]

    def _make_object_name(self, *args):
        path = args[:-1]
        dir = os.path.join(*path)
        if not os.path.exists(dir):
            os.makedirs(dir)
        file = args[-1]+self.obj_extension
        filename = os.path.join(dir, file)
        filename = winpath(filename, self.winonly)
        filename = _qp(filename)
        return filename

    def compile(
        self, sources,
        output_dir=None, macros=None, include_dirs=None, debug=0,
        extra_preargs=None, extra_postargs=None, depends=None
    ):
        macros = macros or []
        include_dirs = include_dirs or []
        extra_preargs = extra_preargs or []
        extra_postargs = extra_postargs or []

        pythonVersionOpts = self.versionOpts()

        if not os.path.exists(output_dir):
            os.makedirs(output_dir)

        binpath = _qp(self._binpath)
        if self.build_exe:
            compileOpts = self._exeCompileOpts
        else:
            compileOpts = self._compileOpts
        outputOpts = self._outputOpts

        includePathOpts = []

        # All object files will be placed in one of three directories:
        # infra   - All of the infrastructure's object files.
        # project - The project's own object files.
        # outside - Any source files specified by the project which are not
        #           contained in the project's own directory.
        orig_sources = sources
        sources = []
        for source in orig_sources:
            if os.path.abspath(source).startswith(os.getcwd()):
                sources.append((winpath(source,self.winonly), 'project'))
            else:
                sources.append((winpath(source, self.winonly), 'outside'))

        if self.with_pyd:
            for file in _pydFiles:
                filePath = os.path.join(_infraDir, 'pyd', file)
                if not os.path.isfile(filePath):
                    raise DistutilsPlatformError("Required Pyd source file '%s' is"
                        " missing." % filePath
                    )
                sources.append((winpath(filePath,self.winonly), 'infra'))
            for file in _utilFiles:
                filePath = os.path.join(_infraDir, 'util', file)
                if not os.path.isfile(filePath):
                    raise DistutilsPlatformError("Required util source file '%s' is"
                        " missing." % filePath
                    )
                sources.append((winpath(filePath,self.winonly), 'infra'))
        if self.build_deimos:
            for file in _deimosFiles:
                filePath = os.path.join(_infraDir, 'deimos', 'python', file)
                if not os.path.isfile(filePath):
                    raise DistutilsPlatformError("Required deimos header "
                        "file '%s' is missing." % filePath
                    )
                sources.append((winpath(filePath,self.winonly), 'infra'))
        # If using PydMain, parse the template file
        if self.build_exe:
            pass
        elif self.with_main:
            name = self.proj_name
            # Store the finished pydmain.d file alongside the object files
            infra_output_dir = winpath(os.path.join(output_dir, 'infra'), self.winonly)
            if not os.path.exists(infra_output_dir):
                os.makedirs(infra_output_dir)
            mainFilename = os.path.join(infra_output_dir, 'pydmain.d')
            make_pydmain(mainFilename, name)
            sources.append((winpath(mainFilename,self.winonly), 'infra'))
        # Add the infraDir to the include path for pyd and utils.
        includePathOpts += self._includeOpts
        includePathOpts[-1] = includePathOpts[-1] % winpath(os.path.join(_infraDir), self.winonly)

        for include_dir in include_dirs:
            includePathOpts += self._includeOpts
            includePathOpts[-1] %= winpath(include_dir, self.winonly)

        if self.build_exe:
            pass
        else:
            # Add DLL/SO boilerplate code file.
            if _isPlatWin:
                boilerplatePath = os.path.join(_infraDir, 'd',
                    'python_dll_windows_boilerplate.d'
                )
            else:
                boilerplatePath = os.path.join(_infraDir, 'd',
                    'python_so_linux_boilerplate.d'
                )
            if not os.path.isfile(boilerplatePath):
                raise DistutilsFileError('Required supporting code file "%s"'
                    ' is missing.' % boilerplatePath
                )
            sources.append((winpath(boilerplatePath,self.winonly), 'infra'))

        for imp in self.string_imports_from_ext:
            if not (os.path.isfile(imp) or os.path.isdir(imp)):
                raise DistutilsFileError('String import file "%s" does not exist' %
                        imp)

        userVersionAndDebugOpts = (
              [self._versionOpt % v for v in self.version_flags_from_ext] +
              [self._debugOpt   % v for v in self.debug_flags_from_ext]
        )

        # Optimization opts
        if debug:
            optimizationOpts = self._debugOptimizeOpts
        elif self.optimize:
            optimizationOpts = self._releaseOptimizeOpts
        else:
            optimizationOpts = self._defaultOptimizeOpts

        unittestOpt = []
        if self.unittest_from_ext:
            unittestOpt.append(self._unittestOpt)
        if self.property_from_ext:
            unittestOpt.append(self._propertyOpt)
        if self.string_imports_from_ext:
            imps = set()
            for imp in self.string_imports_from_ext:
                if os.path.isfile(imp):
                    imps.add(os.path.dirname(os.path.abspath(imp)))
                else:
                    imps.add(os.path.abspath(imp))
            unittestOpt.extend([self._stringImportOpt % (imp,)
                for imp in imps])
        objFiles = []

        if self.lump:
            objName = self._make_object_name(output_dir, 'infra', 'temp')
            outOpts = outputOpts[:]
            outOpts[-1] = outOpts[-1] % objName
            cmdElements = (
                [binpath] + extra_preargs + unittestOpt + compileOpts +
                pythonVersionOpts + optimizationOpts +
                includePathOpts + outOpts + userVersionAndDebugOpts +
                [_qp(source[0]) for source in sources] + extra_postargs
            )
            cmdElements = [el for el in cmdElements if el]
            wrapped_spawn(self,cmdElements, 'pyd_compile')
            return [objName]
        else:
          for source, source_type in sources:
            outOpts = outputOpts[:]
            objFilename = os.path.splitext(source)[0]
            if source_type == 'project':
                objName = self._make_object_name(output_dir, 'project', objFilename)
            elif source_type == 'outside':
                objName = self._make_object_name(output_dir, 'outside', os.path.basename(objFilename))
            else: # infra
                objName = self._make_object_name(output_dir, 'infra', os.path.basename(objFilename))
            objFiles.append(objName)
            outOpts[-1] = outOpts[-1] % objName

            cmdElements = (
                [binpath] + extra_preargs + unittestOpt + compileOpts +
                pythonVersionOpts + optimizationOpts +
                includePathOpts + outOpts + userVersionAndDebugOpts +
                [_qp(source)] + extra_postargs
            )
            cmdElements = [el for el in cmdElements if el]
            spawn0(self,cmdElements)
        return objFiles

    def link (self,
        target_desc, objects, output_filename,
        output_dir=None,
        libraries=None, library_dirs=None, runtime_library_dirs=None,
        export_symbols=None, debug=0,
        extra_preargs=None, extra_postargs=None,
        build_temp=None, target_lang=None
    ):
        # Distutils defaults to None for "unspecified option list"; we want
        # empty lists in that case (this substitution is done here in the body
        # rather than by changing the default parameters in case distutils
        # passes None explicitly).
        libraries = libraries or []
        library_dirs = library_dirs or []
        runtime_library_dirs = runtime_library_dirs or []
        export_symbols = export_symbols or []
        extra_preargs = extra_preargs or []
        extra_postargs = extra_postargs or []

        binpath = self._binpath
        if hasattr(self, '_linkOutputOpts'):
            outputOpts = self._linkOutputOpts[:]
        else:
            outputOpts = self._outputOpts[:]
        objectOpts = [_qp(fn) for fn in objects]

        (objects, output_dir) = self._fix_object_args (objects, output_dir)
        (libraries, library_dirs, runtime_library_dirs) = \
            self._fix_lib_args (libraries, library_dirs, runtime_library_dirs)
        if runtime_library_dirs:
            self.warn('This CCompiler implementation does nothing with'
                ' "runtime_library_dirs": ' + str(runtime_library_dirs)
            )

        if output_dir and os.path.basename(output_filename) == output_filename:
            output_filename = os.path.join(output_dir, output_filename)
        else:
            if not output_filename:
                raise DistutilsFileError( 'Neither output_dir nor' \
                    ' output_filename was specified.')
            output_dir = os.path.dirname(output_filename)
            if not output_dir:
                raise DistutilsFileError( 'Unable to guess output_dir on the'\
                    ' bases of output_filename "%s" alone.' % output_filename)

        # Format the output filename option
        # (-offilename in DMD, -o filename in GDC, -of=filename in LDC)
        outputOpts[-1] = outputOpts[-1] % _qp(winpath(output_filename,self.winonly))

        if not os.path.exists(output_dir):
            os.makedirs(output_dir)

        if not self._need_link(objects, output_filename):
            print ("All binary output files are up to date.")
            return

        if self.build_exe:
            sharedOpts = []
            linkOpts = self._exeLinkOpts
            pythonLibOpt = []
            if target_desc != cc.CCompiler.EXECUTABLE:
                raise LinkError('This CCompiler implementation should be building'
                    ' an executable'
                )
        else:
            # The .def file (on Windows) or -shared and -soname (on Linux)
            sharedOpts = self._def_file(build_temp, output_filename)
            if target_desc != cc.CCompiler.SHARED_OBJECT:
                raise LinkError('This CCompiler implementation should be building '
                    ' a shared object'
                )
            linkOpts = self._linkOpts
        # The python .lib file, if needed
        pythonLibOpt = self._lib_file(libraries)
        if pythonLibOpt:
            pythonLibOpt = _qp(pythonLibOpt)


        # Library linkage options
        print ("library_dirs: %s" % (library_dirs,))
        print ("runtime_library_dirs: %s" % (runtime_library_dirs,))
        print ("libraries: %s"% (libraries,))
        libOpts = gen_lib_options(self, library_dirs, runtime_library_dirs, libraries)

        # Optimization opts
        if debug:
            optimizationOpts = self._debugOptimizeOpts
        elif self.optimize:
            optimizationOpts = self._releaseOptimizeOpts
        else:
            optimizationOpts = self._defaultOptimizeOpts

        cmdElements = (
            [binpath] + extra_preargs + linkOpts + optimizationOpts +
            outputOpts + [pythonLibOpt] + objectOpts + libOpts + sharedOpts +
            extra_postargs
        )
        cmdElements = [el for el in cmdElements if el]

        wrapped_spawn(self,cmdElements, 'pyd_link')

class DMDDCompiler(DCompiler):
    compiler_type = 'dmd'

    executables = {
        'preprocessor' : None,
        'compiler'     : ['dmd', 'dmd.bat'],
        'compiler_so'  : ['dmd', 'dmd.bat'],
        'linker_so'    : ['dmd', 'dmd.bat'],
        'linker_exe'   : ['dmd', 'dmd.bat'],
    }

    _env_var = 'DMD_BIN'

    def _initialize(self):
        self.winonly = True
        # _compileOpts
        self._exeCompileOpts = ['-c']
        self._compileOpts = ['-c']
        if not _isPlatWin: self._compileOpts.append('-fPIC')
        # _outputOpts
        self._outputOpts = ['-of%s']
        # _linkOpts
        if is_posix_static_python():
            self._exeLinkOpts = ['-L'+posix_static_python_lib()]
            self._exeLinkOpts.extend([
                '-L'+l for l in posix_static_python_opts()
            ])
        else:
            self._exeLinkOpts = []
        if _isPlatWin:
            self._linkOpts = []
        elif _isPlatOSX:
            self._linkOpts = ['-shared', '-L'+posix_static_python_lib()]
        else:
            self._linkOpts = ['-shared','-defaultlib=libphobos2.so']
        # _includeOpts
        self._includeOpts = ['-I%s']
        # _versionOpt
        self._versionOpt = '-version=%s'
        # _debugOpt
        self._debugOpt = '-debug=%s'
        # _defaultOptimizeOpts
        self._defaultOptimizeOpts = ['-debug']
        # _stringImportOpt
        self._stringImportOpt = '-J%s'
        # _propertyOpt
        self._propertyOpt = '-property'
        # _unittestOpt
        self._unittestOpt = '-unittest'
        # _debugOptimizeOpts
        self._debugOptimizeOpts = self._defaultOptimizeOpts + [self._unittestOpt, '-g']
        # _releaseOptimizeOpts
        self._releaseOptimizeOpts = ['-version=Optimized', '-release', '-O', '-inline']

    #def link_opts(self,

    def _def_file(self, output_dir, output_filename):
        if _isPlatWin:
            # Automatically create a .def file:
            defFilePath = os.path.join(output_dir, 'infra', 'python_dll_def.def')
            make_pyddef(
                defFilePath,
                os.path.basename(output_filename)
            )
            return [winpath(defFilePath,self.winonly)]
        else:
            return []

    def _lib_file(self, libraries):
        if _isPlatWin:
            # The DMD-compatible .lib file can be generated with implib.exe
            # (from the Digital Mars "Basic Utilities" package) using a command
            # series similar to the following:
            #   cd C:\Windows\system32
            #   \path\to\dm\bin\implib.exe /system python24_digitalmars.lib python24.dll
            #
            # I chose not to incorporate automatic .lib generation into this
            # code because Python X.Y releases are fairly infrequent, so it's
            # more convenient to distribute a pre-extracted .lib file to the
            # users and spare them the need for the "Basic Utilities" package.
            pythonDMDLibPath = _qp(os.path.join(_infraDir, 'windows',
                'python%s_digitalmars.lib' % _pyVerXY
            ))
            if not os.path.isfile(pythonDMDLibPath):
                raise DistutilsFileError('The DMD-compatible Python .lib file'
                    ' which should be located at "%s" is missing.  Try'
                    ' downloading a more recent version of celeriD that'
                    ' contains a .lib file appropriate for your Python version.'
                    % pythonDMDLibPath
                )
            pythonLibOpt = _qp(winpath(pythonDMDLibPath,self.winonly))

            # distutils will normally request that the library 'pythonXY' be
            # linked against.  Since D requires a different .lib file from the
            # one used by the C compiler that built Python, and we've just
            # dealt with that requirement, we take the liberty of removing the
            # distutils-requested pythonXY.lib.
            if 'python' + _pyVerXY in libraries:
                libraries.remove('python' + _pyVerXY)
            if False and _isPlatCygwin and 'python' + _pyVerXDotY  in libraries:
                libraries.remove('python' + _pyVerXDotY)
            return pythonLibOpt
        else:
            return ''

    def library_dir_option(self, dir):
        return '-L-L' + dir

    def runtime_library_dir_option(self, dir):
        if not _isPlatWin:
            return '-L-R' + dir
        else:
            self.warn("Don't know how to set runtime library search path for DMD on Windows.")

    def library_option(self, lib):
        if _isPlatWin:
            return self.library_filename(lib)
        else:
            return '-L-l' + lib
    def compile(self, *args, **kwargs):
        if not _isPlatWin and not self.build_exe:
            output_dir = kwargs.get('output_dir', '')
            if not os.path.exists(os.path.join(output_dir,'infra')):
                os.makedirs(os.path.join(output_dir,'infra'))
            src = os.path.join(_infraDir, 'd', 'so_ctor.c')
            dsto = os.path.join(output_dir, 'infra', 'so_ctor.o')
            spawn0(self,['gcc','-c', src, '-fPIC','-o',dsto])
            self._dmd_so_ctor = dsto
        return DCompiler.compile(self, *args, **kwargs)
    def link (self, *args, **kwargs):
        if not _isPlatWin and not self.build_exe:
            args[1].append(self._dmd_so_ctor)
        return DCompiler.link(self, *args, **kwargs)

class GDCDCompiler(DCompiler):
    compiler_type = 'gdc'

    executables = {
        'preprocessor' : None,
        'compiler'     : ['gdc'],
        'compiler_so'  : ['gdc'],
        'linker_so'    : ['gdc'],
        'linker_exe'   : ['gdc'],
    }

    _env_var = 'GDC_BIN'

    def _initialize(self):
        self._exeCompileOpts = ['-c']
        # _compileOpts
        self._compileOpts = ['-fPIC', '-c']
        # _outputOpts
        self._outputOpts = ['-o', '%s']
        if is_posix_static_python():
            self._exeLinkOpts = posix_static_python_opts()
            self._exeLinkOpts.append(posix_static_python_lib())
        else:
            self._exeLinkOpts = []
        # _linkOpts
        self._linkOpts = ['-fPIC', '-nostartfiles', '-shared']
        # _includeOpts
        self._includeOpts = ['-I', '%s']
        # _versionOpt
        self._versionOpt = '-fversion=%s'
        # _debugOpt
        self._debugOpt = '-fdebug=%s'
        # _defaultOptimizeOpts
        self._defaultOptimizeOpts = ['-fdebug']
        # _stringImportOpt
        self._stringImportOpt = '-J%s'
        # _propertyOpt
        self._propertyOpt = '-fproperty'
        # _unittestOpt
        self._unittestOpt = '-funittest'
        # _debugOptimizeOpts
        self._debugOptimizeOpts = self._defaultOptimizeOpts + ['-g', self._unittestOpt]
        # _releaseOptimizeOpts
        self._releaseOptimizeOpts = ['-fversion=Optimized', '-frelease', '-O3']
    def compile(self, *args, **kwargs):
        if not self.build_exe:
            print("gdc does not support building shared libraries, so python extensions are out of the question")
            sys.exit(1)
        return DCompiler.compile(self, *args, **kwargs)

    def _def_file(self, output_dir, output_filename):
        return ['-Wl,-soname,' + os.path.basename(output_filename)]

    def library_dir_option(self, dir):
        return '-L' + dir

    def runtime_library_dir_option(self, dir):
        return '-Wl,-R' + dir

    def library_option(self, lib):
        return '-l' + lib

class LDCDCompiler(DCompiler):
    compiler_type = 'ldc'
    linker_type = 'gcc'

    executables = {
        'preprocessor' : None,
        'compiler'     : ['ldc2'],
        'compiler_so'  : ['ldc2'],
        'linker_so'    : ['ldc2'],
        'linker_exe'   : ['ldc2'],
    }

    # this is not a env! (but it isn't GDC)
    _env_var = 'LDC_BIN'

    def _initialize(self):
        self._exeCompileOpts = ['-c']
        # _compileOpts
        self._compileOpts = ['-relocation-model=pic', '-c']
        # _outputOpts
        self._outputOpts = ['-of', '%s']
        self._linkOutputOpts = ['-of', '%s']
        # bloody ubuntu has to make things difficult
        if is_posix_static_python():
            self._exeLinkOpts = ['-L'+l for l in posix_static_python_opts()]
            self._exeLinkOpts.append(posix_static_python_lib())
        else:
            self._exeLinkOpts = []
        # _linkOpts
        self._SharedLinkOpts = ['-shared']
        if _isPlatOSX:
            self._SharedLinkOpts.append('-L'+posix_static_python_lib())
        # _includeOpts
        self._includeOpts = ['-I', '%s']
        # _versionOpt
        self._versionOpt = '-d-version=%s'
        # _debugOpt
        self._debugOpt = '-d-debug=%s'
        # _defaultOptimizeOpts
        self._defaultOptimizeOpts = ['-d-debug']
        # _stringImportOpt
        self._stringImportOpt = '-J=%s'
        # _propertyOpt
        self._propertyOpt = '-property'
        # _unittestOpt
        self._unittestOpt = '-unittest'
        # _debugOptimizeOpts
        self._debugOptimizeOpts = self._defaultOptimizeOpts + ['-g', self._unittestOpt]
        # _releaseOptimizeOpts
        self._releaseOptimizeOpts = ['-d-version=Optimized', '-release', '-O3']

    def init_d_opts(self, cmd, ext):
        DCompiler.init_d_opts(self,cmd, ext)
        if self.lump:
            if self.build_exe and '-singleobj' not in self._exeCompileOpts:
                self._exeCompileOpts.append('-singleobj')
            elif not self.build_exe and '-singleobj' not in self._compileOpts:
                self._compileOpts.append('-singleobj')
    def _def_file(self, output_dir, output_filename):
        return []

    def library_dir_option(self, dir):
        return "-L-L" + dir

    def runtime_library_dir_option(self, dir):
        return '-Wl,-R' + dir

    def library_option(self, lib):
        return "-L-l" + lib
    def link (self, *args, **kwargs):
        target_desc = args[0]
        if target_desc == cc.CCompiler.SHARED_OBJECT:
            self._binpath = self.executables['linker_so'][0]
            self._linkOpts = self._SharedLinkOpts
        elif target_desc == cc.CCompiler.EXECUTABLE:
            self._binpath = self.executables['linker_exe'][0]
            self._linkOpts = self._exeLinkOpts
            self._linkOutputOpts = self._outputOpts
        else:
            raise LinkError('This CCompiler implementation does not know'
                ' how to link anything except an extension module (that is, a'
                ' shared object file).'
            )
        return DCompiler.link(self, *args, **kwargs)


# Utility functions:
def _findInPath(fileName, startIn=None):
    # Find a file named fileName in the PATH, starting in startIn.
    try:
        path = os.environ['PATH']
    except KeyError:
        pass
    else:
        pathDirs = path.split(os.pathsep)
        if startIn:
            if startIn in pathDirs:
                pathDirs.remove(startIn)
            pathDirs.insert(0, startIn)

        for pd in pathDirs:
            tentativePath = os.path.join(pd, fileName)
            if os.path.isfile(tentativePath):
                return tentativePath

    return None


def _qp(path): # If path contains any whitespace, quote it.
    if len(path.split()) == 1:
        return path
    else:
        return '"%s"' % path



# Note:  This setup.py is for CeleriD itself, not for an extension written in
# D.

import distutils.core, os, sys
import os.path

import build_manifest

PACKAGE_NAME = 'pyd'

isSourceDist = 'sdist' in [arg.lower() for arg in sys.argv]

f = open('MANIFEST', 'w')
try:
    build_manifest.buildManifest(f, True)
finally:
    f.close()

includedPaths, excludedPaths = build_manifest.listFiles(isSourceDist)

allFiles = [
    build_manifest.convertPathToDistutilsStandard(path)
    for path in includedPaths
]

# Only Python code files *within the pyd package* should go into
# packageFiles (Python code files in examples shouldn't).  A module named
# 'X.py' should later appear in packageModules as 'pyd.X'.
packageCodeFiles = [f for f in allFiles if f.endswith('.py') and '/' not in f]
packageDataFiles = [f for f in allFiles if f not in packageCodeFiles]

packageModules = [
    PACKAGE_NAME + '.' + os.path.splitext(f)[0]
    for f in packageCodeFiles
]

README="""
PyD

PyD provides seamless interoperability between python and the D programming language

Project at https://github.com/ariovistus/pyd

Docs at http://pyd.readthedocs.org/

more about D: http://dlang.org/
"""

distutils.core.setup(
    name=PACKAGE_NAME,
    package_dir={PACKAGE_NAME: os.curdir},
    packages=[PACKAGE_NAME],
    package_data={PACKAGE_NAME: packageDataFiles},
    py_modules=packageModules,

    version=open('version.txt').read().strip(),
    url='https://github.com/ariovistus/pyd',
    description="Interoperability between python and the D programming language",
    long_description=README,
    maintainer='Ellery Newcomer',
    maintainer_email='ellery-newcomer@utulsa.edu',
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX :: Linux',
        #'Programming Language :: D',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ]
)




from distutils.errors import DistutilsPlatformError
import sys
from pyd.dcompiler import _infraDir
import os.path

def make_pydmain(outputFile, projname):
    mainTemplatePath = os.path.join(_infraDir, 'd', 'pydmain_template.d')
    if not os.path.isfile(mainTemplatePath):
        raise DistutilsPlatformError(
            "Required supporting code file %s is missing." % mainTemplatePath
        )
    mainTemplate = open(mainTemplatePath).read()
    mainFileContent = mainTemplate % {'modulename' : projname}
    mainFile = open(outputFile, 'w')
    mainFile.write(mainFileContent)
    mainFile.close()

def make_pyddef(outputFile, projname):
    defTemplatePath = os.path.join(_infraDir, 'd',
        'python_dll_def.def_template'
    )
    if not os.path.isfile(defTemplatePath):
        raise DistutilsFileError('Required def template file "%s" is'
            ' missing.' % defTemplatePath
        )
    f = open(defTemplatePath)
    try:
        defTemplate = f.read()
    finally:
        f.close()
    defFileContent = defTemplate % projname
    f = open(outputFile, 'w')
    try:
        f.write(defFileContent)
    finally:
        f.close()

if __name__ == '__main__':
    make_pydmain(sys.argv[2], sys.argv[1])
    make_pyddef(sys.argv[3], sys.argv[1])

